package jp.co.glossom.adfurikun

import android.content.Context
import android.view.View
import android.widget.FrameLayout
import io.flutter.plugin.common.*
import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory
import jp.tjkapp.adfurikunsdk.moviereward.*

/**
 * ネイティブ広告
 */
class AdfurikunNativeAdFactory(private val messenger: BinaryMessenger, private val channel : MethodChannel) : PlatformViewFactory(StandardMessageCodec.INSTANCE) {

    private var nativeAdViewMap = hashMapOf<Int, AdfurikunNativeAdView>()

    override fun create(context: Context?, viewId: Int, args: Any?): PlatformView {
        return AdfurikunNativeAdView(context, messenger, args, viewId).apply {
            convertViewNumber(args)?.let { number ->
                nativeAdViewMap[number] = this
            }
        }
    }

    fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        convertViewNumber(call.arguments)?.let { number ->
            nativeAdViewMap[number]?.onMethodCall(call, result)
        }
    }

    private fun convertViewNumber(args: Any?): Int? {
        val arguments = (args as? HashMap<String, *>)
        return (arguments?.get(Constants.KEY_NUMBER) as? Int)
    }

    inner class AdfurikunNativeAdView internal constructor(context: Context?, messenger: BinaryMessenger?, args: Any?, private var id: Int)
        : PlatformView, MethodChannel.MethodCallHandler {

        private var nativeAd: AdfurikunNativeAd? = null
        private var loadListener: AdfurikunNativeAdLoadListener? = null
        private var playListener: AdfurikunNativeAdVideoListener? = null

        private var parentView: FrameLayout? = null

        private var number = -1
        private var appId = ""
        private var width = 0
        private var height = 0

        init {
            context?.let {
                parentView = FrameLayout(it)
            }
            val arguments = (args as? HashMap<String, *>)
            number = (arguments?.get(Constants.KEY_NUMBER) as? Int) ?: -1
            appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
            width = (arguments?.get(Constants.KEY_WIDTH) as? Int) ?: 320
            height = (arguments?.get(Constants.KEY_HEIGHT) as? Int) ?: 180
            init(args)
        }

        override fun getView(): View? {
            return parentView
        }

        override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
            when (call.method) {
                Constants.INIT_NATIVE_AD -> init(call.arguments)
                Constants.LOAD_NATIVE_AD -> {
                    if (isTargetAppId(call)) {
                        val timeout = AdfurikunCommons.getLoadWithTimeout(call.arguments as? HashMap<String, *>)
                        if (timeout > 0) {
                            nativeAd?.loadWithTimeout(timeout)
                        } else {
                            nativeAd?.load()
                        }
                    }
                }
                Constants.PLAY_NATIVE_AD -> {
                    if (isTargetAppId(call)) {
                        nativeAd?.nativeAdView?.let {
                            if (it.parent == null) {
                                parentView?.addView(it)
                            }
                        }
                        nativeAd?.play()
                    }
                }
                Constants.DESTROY_NATIVE_AD -> {
                    if (isTargetAppId(call)) {
                        dispose()
                    }
                }
                else -> result.notImplemented()
            }
        }

        /**
         * 初期化
         * @param args
         */
        private fun init(args: Any?) {
            val arguments = (args as? HashMap<String, *>)
            val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
            AdfurikunCommons.setPluginVersion(arguments)
            if (appId.isNotBlank() && this.appId == appId) {
                if (nativeAd == null) {
                    (arguments?.get(Constants.KEY_WIDTH) as? Int)?.let {
                        if (it > 0) {
                            width = it
                        }
                    }
                    (arguments?.get(Constants.KEY_HEIGHT) as? Int)?.let {
                        if (it > 0) {
                            height = it
                        }
                    }
                    var width = 320
                    var height = 180
                    AdfurikunPlugin.currentActivity?.let { activity ->
                        width = AdfurikunCommons.convertDpToPixel(activity, this.width).toInt()
                        height = AdfurikunCommons.convertDpToPixel(activity, this.height).toInt()
                    }
                    nativeAd = AdfurikunNativeAd(AdfurikunPlugin.currentActivity, appId, width, height).apply {
                        setAdfurikunNativeAdLoadListener(createLoadListener)
                        setAdfurikunNativeAdVideoListener(createPlayListener)
                    }
                }
            }
        }

        /**
         * 対象AppId判定
         * @param call
         */
        private fun isTargetAppId(call: MethodCall): Boolean {
            val arguments = (call.arguments as? HashMap<String, *>)
            val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
            return (appId.isNotBlank() && appId == this.appId)
        }

        override fun dispose() {
            nativeAd?.onDestroy()
            nativeAd = null
            loadListener = null
            playListener = null
            nativeAdViewMap.remove(number)
        }

        /**
         * 読み込みリスナーを作成
         */
        private val createLoadListener: AdfurikunNativeAdLoadListener?
            get() {
                loadListener ?: run {
                    loadListener = object : AdfurikunNativeAdLoadListener {
                        override fun onNativeAdLoadFinish(adInfo: AdfurikunNativeAdInfo?, appId: String?) {
                            adInfo?.let {
                                channel.invokeMethod(Constants.ON_NATIVE_AD_LOAD_FINISH,
                                        AdfurikunCommons.getArgs(number = number, appId = appId ?: "", adNetworkKey = it.adNetworkKey, title = it.title, description = it.description))
                            }
                        }

                        override fun onNativeAdLoadError(error: AdfurikunMovieError?, appId: String?) {
                            channel.invokeMethod(Constants.ON_NATIVE_AD_LOAD_ERROR, AdfurikunCommons.getArgs(number = number, appId = appId ?: "", errorCode = error?.errorCode ?: 0))
                        }
                    }
                }
                return loadListener
            }

        /**
         * 再生リスナーを作成
         */
        private val createPlayListener: AdfurikunNativeAdVideoListener?
            get() {
                playListener ?: run {
                    playListener = object : AdfurikunNativeAdVideoListener {
                        override fun onNativeAdViewPlayStart(appId: String?) {
                            channel.invokeMethod(Constants.ON_NATIVE_AD_VIEW_PLAY_START, AdfurikunCommons.getArgs(number = number, appId = appId ?: ""))
                        }

                        override fun onNativeAdViewPlayFinish(appId: String?, isVideoAd: Boolean) {
                            channel.invokeMethod(Constants.ON_NATIVE_AD_VIEW_PLAY_FINISH, AdfurikunCommons.getArgs(number = number, appId = appId ?: ""))
                        }

                        override fun onNativeAdViewPlayFail(appId: String?, error: AdfurikunMovieError?) {
                            channel.invokeMethod(Constants.ON_NATIVE_AD_VIEW_PLAY_FAIL, AdfurikunCommons.getArgs(number = number, appId = appId ?: "", errorCode = error?.errorCode ?: 0))
                        }

                        override fun onNativeAdViewClicked(appId: String?) {
                            channel.invokeMethod(Constants.ON_NATIVE_AD_VIEW_CLICKED, AdfurikunCommons.getArgs(number = number, appId = appId ?: ""))
                        }
                    }
                }
                return playListener
            }
    }
}