import Foundation
import Flutter
import ADFMovieReward

// MARK: FullscreenAdDelegate Class

class FullscreenAdDelegate: NSObject, ADFmyMovieRewardDelegate {
    private let channel: FlutterMethodChannel
    private let number: Int
    
    init(_ channel: FlutterMethodChannel, _ number: Int) {
        self.channel = channel
        self.number = number
    }
    
    private func args(number: Int,
                      appId: String,
                      adNetworkKey: String? = nil,
                      errorCode: Int? = nil) -> Dictionary<String, Any?> {
        return [
            ADFConst.KEY_NUMBER: number,
            ADFConst.KEY_APP_ID: appId,
            ADFConst.KEY_AD_NETWORK_KEY: adNetworkKey,
            ADFConst.KEY_ERROR_CODE: errorCode
        ]
    }
    
    private func invokeMethod(_ method: String, _ args: Dictionary<String, Any?>) {
        channel.invokeMethod(method, arguments: args)
    }
    
    // MARK: ADF Delegate Method
    
    func adsFetchCompleted(_ appID: String!, isTestMode isTestMode_inApp: Bool, isManualMode: Bool) {
        if let appId = appID {
            print("[ADF DEBUG] call adsFetchCompleted, appId:\(appId), number:\(number)")
            let args: Dictionary<String, Any?> = args(number: number,
                                                      appId: appId)
            invokeMethod(ADFConst.CALLBACK_ON_PREPARE_SUCCESS, args)
        } else {
            print("[ADF DEBUG] call adsFetchCompleted appID is nil, number:\(number)")
        }
    }
    
    func adsFetchFailed(_ appID: String!, error: Error!) {
        if let appId = appID {
            var errorCode: Int? = nil
            if let err = error as NSError? {
                print("[ADF DEBUG] call adsFetchFailed, appId:\(appId), number:\(number), error:\(err.localizedDescription)")
                errorCode = err.code
            }
            let args: Dictionary<String, Any?> = args(number: number,
                                                      appId: appId,
                                                      errorCode: errorCode)
            invokeMethod(ADFConst.CALLBACK_ON_PREPARE_FAILURE, args)
        } else {
            print("[ADF DEBUG] call adsFetchFailed appId is nil, number:\(number)")
        }
    }
    
    public func adsDidShow(_ appID: String!, adNetworkKey: String!) {
        if let appId = appID {
            print("[ADF DEBUG] call adsDidShow, appId:\(appId), number:\(number)")
            let args: Dictionary<String, Any?> = args(number: number,
                                                      appId: appId,
                                                      adNetworkKey: adNetworkKey)
            invokeMethod(ADFConst.CALLBACK_ON_START_PLAYING, args)
        } else {
            print("[ADF DEBUG] call adsDidShow appID is nil, number:\(number)")
        }
    }
    
    public func adsDidCompleteShow(_ appID: String!) {
        if let appId = appID {
            print("[ADF DEBUG] call adsDidCompleteShow, appId:\(appId), number:\(number)")
            let args: Dictionary<String, Any?> = args(number: number,
                                                      appId: appId)
            invokeMethod(ADFConst.CALLBACK_ON_FINISHED_PLAYING, args)
        } else {
            print("[ADF DEBUG] call adsDidCompleteShow appID is nil, number:\(number)")
        }
    }
    
    public func adsDidHide(_ appID: String!) {
        if let appId = appID {
            print("[ADF DEBUG] call adsDidHide, appId:\(appId), number:\(number)")
            let args: Dictionary<String, Any?> = args(number: number,
                                                      appId: appId)
            invokeMethod(ADFConst.CALLBACK_ON_AD_CLOSE, args)
        } else {
            print("[ADF DEBUG] call adsDidHide appID is nil, number:\(number)")
        }
    }
    
    public func adsPlayFailed(_ appID: String!) {
        if let appId = appID {
            print("[ADF DEBUG] call adsPlayFailed, appId:\(appId), number:\(number)")
            let args: Dictionary<String, Any?> = args(number: number,
                                                      appId: appId)
            invokeMethod(ADFConst.CALLBACK_ON_FAILED_PLAYING, args)
        } else {
            print("[ADF DEBUG] call adsPlayFailed appID is nil, number:\(number)")
        }
    }
}


// MARK: FullscreenAdManager Class

class FullscreenAdManager: NSObject {
    private let channel: FlutterMethodChannel
    
    private var rewardMap: Dictionary<Int, ADFmyMovieReward> = [:]
    private var interstitialMap: Dictionary<Int, ADFmyInterstitial> = [:]
    
    private var delegateMap: Dictionary<Int, FullscreenAdDelegate> = [:]
    
    init(_ channel: FlutterMethodChannel) {
        self.channel = channel
    }
    
    func checkMethod(_ call: FlutterMethodCall, _ checkString: String) -> Bool {
        return call.method.hasPrefix(checkString + ADFConst.SEPARATOR_METHOD)
    }
    
    func handle(_ call: FlutterMethodCall, _ result: @escaping FlutterResult, _ adType: String) {
        guard let args = call.arguments as? [String: Any?],
              let number = args[ADFConst.KEY_NUMBER] as? Int,
              let appId = args[ADFConst.KEY_APP_ID] as? String else {
            print("[ADF WARNING] FullscreenAdManager:handle args is abnormal")
            return
        }
        if adType == ADFConst.AD_TYPE_REWARD {
            if checkMethod(call, ADFConst.METHOD_INIT) {
                let pluginVersion = args[ADFConst.KEY_PLUGIN_VERSION] as? String
                _ = createRewardIfNeeded(number, appId, pluginVersion)
            } else if checkMethod(call, ADFConst.METHOD_LOAD) {
                let reward = getReward(number)
                if let timeout = args[ADFConst.KEY_TIMEOUT] as? Int32, timeout > 0 {
                    reward?.load(withTimeout: timeout)
                } else {
                    reward?.load()
                }
            } else if checkMethod(call, ADFConst.METHOD_IS_PREPARED) {
                if let reward = getReward(number) {
                    result(reward.isPrepared())
                } else {
                    result(false)
                }
            } else if checkMethod(call, ADFConst.METHOD_PLAY) {
                let reward = getReward(number)
                if let customParams = args[ADFConst.KEY_CUSTOM_PARAMS] as? Dictionary<String, String> {
                    reward?.play(withCustomParam: customParams)
                } else {
                    reward?.play()
                }
            } else if checkMethod(call, ADFConst.METHOD_DESTROY) {
                getReward(number)?.dispose()
                removeReward(number)
            }
        } else if adType == ADFConst.AD_TYPE_INTERSTITIAL {
            if checkMethod(call, ADFConst.METHOD_INIT) {
                let pluginVersion = args[ADFConst.KEY_PLUGIN_VERSION] as? String
                _ = createInterstitialIfNeeded(number, appId, pluginVersion)
            } else if checkMethod(call, ADFConst.METHOD_LOAD) {
                let interstitial = getInterstitial(number)
                if let timeout = args[ADFConst.KEY_TIMEOUT] as? Int32, timeout > 0 {
                    interstitial?.load(withTimeout: timeout)
                } else {
                    interstitial?.load()
                }
            } else if checkMethod(call, ADFConst.METHOD_IS_PREPARED) {
                if let interstitial = getInterstitial(number) {
                    result(interstitial.isPrepared())
                } else {
                    result(false)
                }
            } else if checkMethod(call, ADFConst.METHOD_PLAY) {
                let interstitial = getInterstitial(number)
                if let customParams = args[ADFConst.KEY_CUSTOM_PARAMS] as? Dictionary<String, String> {
                    interstitial?.play(withCustomParam: customParams)
                } else {
                    interstitial?.play()
                }
            } else if checkMethod(call, ADFConst.METHOD_DESTROY) {
                getInterstitial(number)?.dispose()
                removeInterstitial(number)
            }
        }
    }
    
    // MARK: Private Method
    
    private func createRewardIfNeeded(_ number: Int, _ appId: String, _ pluginVersion: String?) -> ADFmyMovieReward {
        if let reward = rewardMap[number] {
            return reward
        }
        let delegate = FullscreenAdDelegate(channel, number)
        delegateMap.updateValue(delegate, forKey: number)
        let option: [AnyHashable: Any] = [ADFConst.KEY_PLUGIN:
                                            [ADFConst.KEY_TYPE: ADFConst.VALUE_FLUTTER,
                                             ADFConst.KEY_VERSION: pluginVersion ?? ""]]
        ADFmyMovieReward.initialize(withAppID: appId, option: option)
        let reward: ADFmyMovieReward = ADFmyMovieReward.getInstance(appId, delegate: delegate)
        rewardMap.updateValue(reward, forKey: number)
        return reward
    }
    
    private func getReward(_ number: Int) -> ADFmyMovieReward? {
        return rewardMap[number]
    }
    
    private func removeReward(_ number: Int) {
        rewardMap.removeValue(forKey: number)
        delegateMap.removeValue(forKey: number)
    }
    
    private func createInterstitialIfNeeded(_ number: Int, _ appId: String, _ pluginVersion: String?) -> ADFmyInterstitial {
        if let interstitial = interstitialMap[number] {
            return interstitial
        }
        let delegate = FullscreenAdDelegate(channel, number)
        delegateMap.updateValue(delegate, forKey: number)
        ADFmyInterstitial.initialize(withAppID: appId, option: [ADFConst.KEY_PLUGIN: pluginVersion ?? ""])
        let interstitial: ADFmyInterstitial = ADFmyInterstitial.getInstance(appId, delegate: delegate)
        interstitialMap.updateValue(interstitial, forKey: number)
        return interstitial
    }
    
    private func getInterstitial(_ number: Int) -> ADFmyInterstitial? {
        return interstitialMap[number]
    }
    
    private func removeInterstitial(_ number: Int) {
        interstitialMap.removeValue(forKey: number)
        delegateMap.removeValue(forKey: number)
    }
}
